#include <unistd.h>
#include <termios.h>
#include <signal.h>
#include <string.h>

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <viper.h>

#include "vwmterm.h"
#include "vscreen.h"
#include "watchdog.h"
#include "events.h"

gint vwmterm_ON_KEYSTROKE(gint32 keystroke,WINDOW *window)
{
	gchar				*sequence=NULL;
	vwmterm_t		*vwmterm;
	struct termios	*term_desc;

	if(keystroke==KEY_MOUSE) return 1;

	viper_thread_enter();
	vwmterm=(vwmterm_t*)viper_window_get_userptr(window);
	viper_thread_leave();

	/*	check foreground job control status.  this was moved from the input
		parsing loop on 5-3-2007.  keystrokes occur much less frequently and
		it's really the only time its necessary.	*/
	vwmterm_watchdog(vwmterm);
	term_desc=&vwmterm->vscreens[vwmterm->scr_idx]->term_desc;

	/*	most applications expect CR not LF	*/
	if(keystroke==10) keystroke=13;

	if(keystroke==KEY_BACKSPACE)
	{
		keystroke=term_desc->c_cc[VERASE];
	}

	/*	some keys require 2-byte
		integers which do not always correspond to the input code/scan code
		expected by the terminal.  In such case we have to do some special
		translation.  This should be rare though.	*/
	if(keystroke==KEY_UP) sequence="\x1B[A";
	if(keystroke==KEY_DOWN) sequence="\x1B[B";
	if(keystroke==KEY_RIGHT) sequence="\x1B[C";
	if(keystroke==KEY_LEFT) sequence="\x1B[D";
	if(keystroke==KEY_F(1)) sequence="\x1B[[A";
	if(keystroke==KEY_F(2)) sequence="\x1B[[B";
	if(keystroke==KEY_F(3)) sequence="\x1B[[C";
	if(keystroke==KEY_F(4)) sequence="\x1B[[D";
	if(keystroke==KEY_F(5)) sequence="\x1B[[E";
	if(keystroke==KEY_F(6)) sequence="\x1B[17~";
	if(keystroke==KEY_F(7)) sequence="\x1B[18~";
	if(keystroke==KEY_F(8)) sequence="\x1B[19~";
	if(keystroke==KEY_F(9)) sequence="\x1B[20~";
	if(keystroke==KEY_F(10)) sequence="\x1B[21~";
	if(keystroke==KEY_F(11)) sequence="\x1B[22~";
	if(keystroke==KEY_F(12)) sequence="\x1B[23~";
	if(keystroke==KEY_HOME) sequence="\x1B[1~";
   if(keystroke==KEY_DC) sequence="\x1B[3~";
	if(keystroke==KEY_END) sequence="\x1B[4~";
	if(keystroke==KEY_NPAGE) sequence="\x1B[6~";
	if(keystroke==KEY_PPAGE) sequence="\x1B[5~";

	/*	a simple key was pressed	*/
	if(((keystroke & 0xFFFFFF00)==0) && sequence==NULL)
	{
		write(vwmterm->master_fd,(gchar*)&keystroke,sizeof(gchar));
      tcdrain(vwmterm->master_fd);
		return 1;
	}

	if(sequence!=NULL)
		write(vwmterm->master_fd,sequence,strlen(sequence));
	else
		write(vwmterm->master_fd,(gchar*)&keystroke,sizeof(keystroke));

   tcdrain(vwmterm->master_fd);

	return 1;
}


gint vwmterm_ON_RESIZE(WINDOW *window,gpointer anything)
{
	vwmterm_t	*vwmterm;
	WINDOW		*vscreen_window;
	gint			array_sz;
	gint			i;

	viper_thread_enter();
	vwmterm=(vwmterm_t*)viper_window_get_userptr(window);
	array_sz=sizeof(vwmterm->vscreens)/sizeof(vwmterm->vscreens[0]);

	getmaxyx(window,vwmterm->win_sz.ws_row,vwmterm->win_sz.ws_col);
	for(i=0;i<array_sz;i++)
	{
		vscreen_window=vwmterm->vscreens[i]->window;
		wresize(vscreen_window,vwmterm->win_sz.ws_row,vwmterm->win_sz.ws_col);
	}
	viper_thread_leave();

	ioctl(vwmterm->slave_fd,TIOCSWINSZ,&vwmterm->win_sz);
	kill(vwmterm->child_pid,SIGWINCH);

	return 0;
}

gint vwmterm_ON_DESTROY(WINDOW *window,gpointer anything)
{
	vwmterm_t	*vwmterm;
	gint			array_sz;
	gint			i;

	vwmterm=(vwmterm_t*)anything;
	array_sz=sizeof(vwmterm->vscreens)/sizeof(vwmterm->vscreens[0]);

	close(vwmterm->master_fd);
	close(vwmterm->slave_fd);
   if(vwmterm->carryover!=NULL) g_free(vwmterm->carryover);

	for(i=0;i<array_sz;i++)
	{
		vscreen_destroy(vwmterm->vscreens[i]);
	}

	g_free(vwmterm);
	return 0;
}

gint vwmterm_ON_CLOSE(WINDOW *window,gpointer anything)
{
	vwmterm_t	*vwmterm;

	vwmterm=(vwmterm_t*)anything;
	kill(vwmterm->child_pid,SIGKILL);
	waitpid(vwmterm->child_pid,NULL,0);

	return VIPER_EVENT_WINDOW_DESIST;
}
