#include <unistd.h>
#include <termios.h>

#include <viper.h>

#include "vscreen.h"
#include "scrollback.h"

static void 	scrollback_activate(vscreen_t *scrollback);
static void		scrollback_move(vscreen_t *scrollback,gint y,gint x);
static void		scrollback_addch(vscreen_t *scrollback,chtype ch);

/* helper funcs	*/
static void		scrollback_scroll_up(vscreen_t *scrollback);

vs_scroll_t* scrollback_vscreen_from(WINDOW *source,guint pty_fd)
{
	vs_scroll_t		*scrollback;
	WINDOW			*window;
	struct termios	*term_desc;

	if(source==NULL) return NULL;
	window=dupwin(source);

	scrollback=(vs_scroll_t*)g_malloc0(sizeof(vs_scroll_t));
	VSCREEN(scrollback)->pty_fd=pty_fd;

	scrollok(window,FALSE);
	idlok(window,FALSE);
	wattrset(window,VIPER_COLORS(COLOR_WHITE,COLOR_BLACK));
	wbkgdset(window,VIPER_COLORS(COLOR_WHITE,COLOR_BLACK));
	werase(window);

	/* read existing settings	*/
	term_desc=&VSCREEN(scrollback)->term_desc;
	tcgetattr(pty_fd,term_desc);

	term_desc->c_lflag |= ISIG | IEXTEN | ICANON | ECHO;
	term_desc->c_iflag |= BRKINT | IGNPAR | IXON | ICRNL;
	term_desc->c_iflag &= ~IXANY & ~IGNCR;
	term_desc->c_oflag |= OPOST | ONLRET | TAB3;
//	term_desc->c_oflag |= OPOST | ONLRET;
//	term_desc->c_oflag &= ~TAB3;
//	term_desc->c_oflag &= ~ONLCR & ~OCRNL & ~ONOCR & ~OFILL & ~TAB3;
	term_desc->c_oflag &= ~ONLCR & ~OCRNL & ~ONOCR & ~OFILL;

#ifdef OLCUC
   term_desc->c_oflag &= ~OLCUC;  /* don't xlate lower case to upper  */
#endif

	VSCREEN(scrollback)->window=window;
	VSCREEN(scrollback)->state |= VSCREEN_SCROLLOK;
	VSCREEN(scrollback)->fg=COLOR_WHITE;
	VSCREEN(scrollback)->bg=COLOR_BLACK;
	VSCREEN(scrollback)->activate=scrollback_activate;
	VSCREEN(scrollback)->write_char=scrollback_addch;
	VSCREEN(scrollback)->move_cursor=scrollback_move;

	return scrollback;
}

static void scrollback_activate(vscreen_t *scrollback)
{
	tcsetattr(scrollback->pty_fd,TCSAFLUSH,&scrollback->term_desc);

	scrollok(scrollback->window,TRUE);
	scrollback->state |= VSCREEN_SCROLLOK;
	scrollback->state &= ~(VSCREEN_CINVIS);
}

static void scrollback_addch(vscreen_t *scrollback,chtype ch)
{
	gint     width,height;
   gint     cur_x,cur_y;

   getmaxyx(scrollback->window,height,width);
   getyx(scrollback->window,cur_y,cur_x);

	if(is_curs_at_lower_right(scrollback->window)!=-1)
	{
		/*	cursor was parked	*/
		if(scrollback->state & VSCREEN_CPARKED)
		{
			scrollback->state &= ~VSCREEN_CPARKED;
			if(ch=='\r')
			{
				scrollback->state &= ~VSCREEN_CWRAPPED;
				wmove(scrollback->window,cur_y,0);
				return;
			}

			scrollback_scroll_up(scrollback);
			wmove(scrollback->window,cur_y,0);
			if(ch>=32)
			{
				waddch(scrollback->window,ch);
				return;
			}
		}
		/* print and park cursor	*/
		else
		{
			if(ch>=32)
			{
				scrollback->state |= VSCREEN_CPARKED;
				scrollback->state |= VSCREEN_CWRAPPED;
				waddch(scrollback->window,ch);
				return;
			}
		}
	}

	if(ch=='\r')
	{
		scrollback->state &= ~VSCREEN_CWRAPPED;
		scrollback->state &= ~VSCREEN_CPARKED;
/*
		if(is_curs_at_lower_left(scrollback->window)!=-1)
		{
			scrollback_scroll_up(scrollback);
		}
*/
		wmove(scrollback->window,cur_y,0);
		return;
	}

	if(ch=='\n')
	{
		scrollback->state &= ~VSCREEN_CPARKED;
		if(scrollback->state & VSCREEN_CWRAPPED)
		{
			scrollback->state &= ~VSCREEN_CWRAPPED;
			return;
		}

		if(is_curs_at_bottom(scrollback->window)!=-1)
		{
			scrollback_scroll_up(scrollback);
			wmove(scrollback->window,cur_y,0);
		}
		else
		{
			wmove(scrollback->window,cur_y+1,0);
		}
		return;
	}

	if(is_curs_at_right(scrollback->window)!=-1)
	{
		if(ch>=32)
		{
			scrollback->state |= VSCREEN_CWRAPPED;
			waddch(scrollback->window,ch);
			return;
		}
	}

	waddch(scrollback->window,ch);
//	mvprintw(1,1,"x=%d y=%d ",cur_x,cur_y);

	return;
}

static void scrollback_move(vscreen_t *scrollback,gint y,gint x)
{
	scrollback->state &= ~(VSCREEN_CWRAPPED);
	scrollback->state &= ~(VSCREEN_CPARKED);
	wmove(scrollback->window,y,x);

	return;
}

static void scrollback_scroll_up(vscreen_t *scrollback)
{
	scrollok(scrollback->window,TRUE);
	wscrl(scrollback->window,1);
	scrollok(scrollback->window,FALSE);

	return;
}

