#include <unistd.h>
#include <termios.h>

#include <viper.h>

#include "vwmterm.h"
#include "watchdog.h"

void vwmterm_watchdog(vwmterm_t *vwmterm)
{
	pid_t	            tcpid;

	/*	check to see if bash has regained control of the
		console.  this is important because some progs send a vt100 code which
		disables scrolling.  when we get control of the console back we need
		to enable scrolling just in case it was disabled	*/
	tcpid=tcgetpgrp(vwmterm->master_fd);
	if(tcpid==-1) return;

	/*	store the value of the old state	*/
	vwmterm->old_state=vwmterm->new_state;

	if(tcpid!=vwmterm->child_pid) vwmterm->new_state &= (~PTY_FG);
	else vwmterm->new_state |= PTY_FG;

	/* these flags are tripped when the foreground process terminates (normally
		or because of a SIGKILL and then bash resumes control as the foreground
		process.  in the case of a KILL signal, ncurses can't issue the escape
		code to notify us to reset the terminal to scrollback mode so we do it
		manually based on this trigger	*/
	if(VWMTERM_IS_FG(vwmterm) && VWMTERM_WAS_BG(vwmterm))
	{
      /* update the terminal description. */
		viper_thread_enter();
		vwmterm_activate_vscreen(vwmterm,SCROLLBACK);
		viper_thread_leave();
	}

	return;
}

